package cn.ninegame.genericframework.basic;

import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.os.Message;
import android.os.SystemClock;
import android.text.TextUtils;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.math.BigInteger;
import java.util.HashMap;
import java.util.Set;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import cn.ninegame.genericframework.module.CacheOperation.MessageOperation;
import cn.ninegame.genericframework.module.IModuleEntry;
import cn.ninegame.genericframework.tools.BundleBuilder;
import cn.ninegame.genericframework.tools.CRC16;
import cn.ninegame.genericframework.tools.LOG;

/**
 * Created by lingcong on 2015/3/19.
 * Changed by zhonghui
 * 职责：
 * 1、组件间通信中间人。
 * 2、为消息寻找正确消息处理者，并获得处理结果
 * 3、传递消息的处理结果给消息发起者
 */
class MsgBroker implements Handler.Callback {

    private final HandlerPoster mainThreadPoster;
    private final BackgroundPoster backgroundPoster;
    private final AsyncPoster asyncPoster;

    private final static ExecutorService DEFAULT_EXECUTOR_SERVICE = Executors.newCachedThreadPool();

    private static final String KEY_MESSAGE_ID = "msg_id";
    private static final String KEY_MESSAGE_BODY = "msg_body";
    private static final String KEY_MESSAGE_LISTENER = "msg_listener";

    private static final int MSG_DEFAULT_ID = 1;
    private static final int MSG_EXTRA_ID = 2;
    private static final String TAG = "MsgBroker";

    private ControllerCenter mControllerCenter = null;
    private HashMap<Integer,String> mMessageHandlerMap = new HashMap<Integer, String>(2);
    private HashMap<Integer,GundamMessage> mMessageDefinitionMap = new HashMap<Integer, GundamMessage>(2);
    private HashMap<String,Integer> mNameToIdsMap = new HashMap<>(2);
    private HashMap<Integer,Method> mMessageMethodMap = new HashMap<Integer, Method>(2);

    private Handler mMainHandler = null;
    private ExecutorService mBackgroundExecutor = DEFAULT_EXECUTOR_SERVICE;

    ExecutorService getExecutorService() {
        return mBackgroundExecutor;
    }
    private boolean checkMainThread;

    public MsgBroker(boolean checkMainThread) {
        this.checkMainThread = checkMainThread;
        if (mMainHandler == null) {
            mMainHandler = new Handler(Looper.getMainLooper(),this);
        }
        mainThreadPoster = new HandlerPoster(this, Looper.getMainLooper(), 10);
        backgroundPoster = new BackgroundPoster(this);
        asyncPoster = new AsyncPoster(this);
        if(this.checkMainThread){
            checkThread();
        }
    }

    public MsgBroker() {
        if (mMainHandler == null) {
            mMainHandler = new Handler(Looper.getMainLooper(),this);
        }
        mainThreadPoster = new HandlerPoster(this, Looper.getMainLooper(), 10);
        backgroundPoster = new BackgroundPoster(this);
        asyncPoster = new AsyncPoster(this);
        checkThread();
    }

    public void checkThread() {
        if(Thread.currentThread().getId() != Looper.getMainLooper().getThread().getId()) {
            final Error error = new Error("can not create msgBroker in work thread");
            mMainHandler.post(new Runnable() {
                @Override
                public void run() {
                    throw error;
                }
            });
        }
    }

    public void setControllerCenter(ControllerCenter center) {
        mControllerCenter = center;
    }

    public synchronized void registerMessageHandler(String messageHandlerID,GundamMessage[] msgMap) {
        if (TextUtils.isEmpty(messageHandlerID)) {
            return;
        }
        if(msgMap == null || msgMap.length == 0) {
            return;
        }

        for (int i = 0; i < msgMap.length ; i++) {
            if(TextUtils.isEmpty(msgMap[i].name)) {
                continue;
            }
            String handlerID = mMessageHandlerMap.get(msgMap[i].id);
            if(!TextUtils.isEmpty(handlerID)) {
                continue;
            }
            mNameToIdsMap.put("#"+msgMap[i].name,msgMap[i].id);
            mMessageDefinitionMap.put(msgMap[i].id,msgMap[i]);
            mMessageHandlerMap.put(msgMap[i].id,messageHandlerID);
        }
    }

    public int getMessageIdByName(String messageName){
        String key = "#"+messageName.toLowerCase();
        if(mNameToIdsMap.containsKey(key)){
            return mNameToIdsMap.get(key);
        }
        statMessageNotFindException(messageName);
        FrameworkService.addDebugLog(TAG, ILogHandler.LogLevel.error,"can getmessageId By Name:"+ key);
        return -1;
    }

    public void sendMessage(int messageId, Bundle messageData){
        if(messageId<=0) return;
        sendMessageForResult(messageId,messageData,null);
    }

    public void sendMessage(int messageId, Bundle messageData,IMessageContext messageContext){
        if(messageId<=0) return;
        sendMessageForResult(messageId,messageData,null,messageContext);
    }

    public void sendMessageForResult(final int messageId, final Bundle messageData, final IResultListener listener) {
        if(messageId<=0) return;
        GundamMessage gundamMessage = mMessageDefinitionMap.get(messageId);
        if(gundamMessage==null){
            //兼容旧的消息机制
            statMessageNotFindException(messageId);
            FrameworkService.addDebugLog(TAG, ILogHandler.LogLevel.error,"sendMessageForResult: can not find event definition for message,id:"+ getLogMessageId(messageId)+ " execute thread:" + (isMainThread()?"main thread":"background thread"));
            gundamMessage = new GundamMessage();
            gundamMessage.id = messageId;
            gundamMessage.threadMode ="posting";
        }
        ////兼容旧的消息机制,默认
        if(TextUtils.isEmpty(gundamMessage.threadMode)){
            gundamMessage.threadMode="posting";
        }
        ThreadMode messageThreadMode = ThreadMode.POSTING;
        if(gundamMessage.threadMode.equals("main")){
            messageThreadMode = ThreadMode.MAIN;
        }else if(gundamMessage.threadMode.equals("background")){
            messageThreadMode = ThreadMode.BACKGROUND;
        }else if(gundamMessage.threadMode.equals("posting")){
            messageThreadMode = ThreadMode.POSTING;
        }else if(gundamMessage.threadMode.equals("async")){
            messageThreadMode = ThreadMode.ASYNC;
        }
        if(FrameworkService.getInstance().mIsDebug){
            String msg = "sendMessageForResult:dispatcher message,id:"+ getLogMessageId(messageId) + (listener!=null?" with result callback ":"");
            if(messageData!=null){
                StringBuffer stringBuffer = new StringBuffer();
                final Set<String> keySet = messageData.keySet();
                for (final String key: keySet) {
                    stringBuffer.append('\"');
                    stringBuffer.append(key);
                    stringBuffer.append("\"=\"");
                    stringBuffer.append(messageData.get(key));
                    stringBuffer.append("\", ");
                }
                msg = msg + " data:"+ stringBuffer.toString();
            }else{
                msg = msg  + " without data ";
            }
            msg += ",thread_mode:" + gundamMessage.threadMode +  ",execute thread: " + (isMainThread()?"main thread":"background thread");
            FrameworkService.addDebugLog(TAG, ILogHandler.LogLevel.info,msg);
        }
        this.enQueue(messageThreadMode,gundamMessage,messageId,messageData,listener,null);
    }

    public void sendMessageForResult(final int messageId, final Bundle messageData, final IResultListener listener,IMessageContext messageContext) {
        if(messageId<=0) return;
        GundamMessage gundamMessage = mMessageDefinitionMap.get(messageId);
        if(gundamMessage==null){
            //兼容旧的消息机制
            statMessageNotFindException(messageId);
            FrameworkService.addDebugLog(TAG, ILogHandler.LogLevel.error,"sendMessageForResult: can not find event definition for message,id:"+ getLogMessageId(messageId)+ " execute thread:" + (isMainThread()?"main thread":"background thread"));
            gundamMessage = new GundamMessage();
            gundamMessage.id = messageId;
            gundamMessage.threadMode ="posting";
        }
        ////兼容旧的消息机制,默认
        if(TextUtils.isEmpty(gundamMessage.threadMode)){
            gundamMessage.threadMode="posting";
        }
        ThreadMode messageThreadMode = ThreadMode.POSTING;
        if(gundamMessage.threadMode.equals("main")){
            messageThreadMode = ThreadMode.MAIN;
        }else if(gundamMessage.threadMode.equals("background")){
            messageThreadMode = ThreadMode.BACKGROUND;
        }else if(gundamMessage.threadMode.equals("posting")){
            messageThreadMode = ThreadMode.POSTING;
        }else if(gundamMessage.threadMode.equals("async")){
            messageThreadMode = ThreadMode.ASYNC;
        }
        if(FrameworkService.getInstance().mIsDebug){
            String msg = "sendMessageForResult:dispatcher message,id:"+ getLogMessageId(messageId) + (listener!=null?" with result callback ":"");
            if(messageData!=null){
                StringBuffer stringBuffer = new StringBuffer();
                final Set<String> keySet = messageData.keySet();
                for (final String key: keySet) {
                    stringBuffer.append('\"');
                    stringBuffer.append(key);
                    stringBuffer.append("\"=\"");
                    stringBuffer.append(messageData.get(key));
                    stringBuffer.append("\", ");
                }
                msg = msg + " data:"+ stringBuffer.toString();
            }else{
                msg = msg  + " without data ";
            }
            msg += ",thread_mode:" + gundamMessage.threadMode +  ",execute thread: " + (isMainThread()?"main thread":"background thread");
            FrameworkService.addDebugLog(TAG, ILogHandler.LogLevel.info,msg);
        }
        this.enQueue(messageThreadMode,gundamMessage,messageId,messageData,listener,messageContext);
    }

    private void enQueue(ThreadMode messageThreadMode,GundamMessage gundamMessage,int messageId,Bundle  messageData,IResultListener listener,IMessageContext messageContext){
        switch (messageThreadMode){
            case MAIN:
                Message message = Message.obtain();
                message.what = MSG_DEFAULT_ID;
                Bundle bundle = new BundleBuilder()
                        .putInt(KEY_MESSAGE_ID, messageId)
                        .putBundle(KEY_MESSAGE_BODY, messageData)
                        .putParcelable(KEY_MESSAGE_LISTENER, listener)
                        .create();
                message.obj = new MessageData(bundle,messageContext);
                if(Thread.currentThread().getId() == Looper.getMainLooper().getThread().getId()) {
                    handleMessage(message);
                } else {
                    mainThreadPoster.enqueue(message, gundamMessage);
                }
                break;
            case BACKGROUND:
                Message messageBackGround = Message.obtain();
                messageBackGround.what = MSG_DEFAULT_ID;
                Bundle bundleGround = new BundleBuilder()
                        .putInt(KEY_MESSAGE_ID, messageId)
                        .putBundle(KEY_MESSAGE_BODY, messageData)
                        .putParcelable(KEY_MESSAGE_LISTENER, listener)
                        .create();
                messageBackGround.obj =  new MessageData(bundleGround,messageContext);
                if(Thread.currentThread().getId() == Looper.getMainLooper().getThread().getId()) {
                    backgroundPoster.enqueue(messageBackGround, gundamMessage);
                } else {
                    handleMessage(messageBackGround);
                }
                break;
            case ASYNC:
                Message messageAsync = Message.obtain();
                messageAsync.what = MSG_DEFAULT_ID;
                Bundle bundleSync = new BundleBuilder()
                        .putInt(KEY_MESSAGE_ID, messageId)
                        .putBundle(KEY_MESSAGE_BODY, messageData)
                        .putParcelable(KEY_MESSAGE_LISTENER, listener)
                        .create();
                messageAsync.obj =  new MessageData(bundleSync,messageContext);
                asyncPoster.enqueue(messageAsync, gundamMessage);
                break;
            case POSTING:
                Message messagePosting = Message.obtain();
                messagePosting.what = MSG_DEFAULT_ID;
                Bundle bundlePosting = new BundleBuilder()
                        .putInt(KEY_MESSAGE_ID, messageId)
                        .putBundle(KEY_MESSAGE_BODY, messageData)
                        .putParcelable(KEY_MESSAGE_LISTENER, listener)
                        .create();
                messagePosting.obj = new MessageData(bundlePosting,messageContext);
                handleMessage(messagePosting);
                break;
        }
    }

    public Bundle sendMessageSync(int messageId){
        return sendMessageSync(messageId,null,null);
    }

    public Bundle sendMessageSync(int messageId, Bundle messageData){
        return sendMessageSync(messageId,messageData,null);
    }

    public Bundle sendMessageSync(int messageId, Bundle messageData,IMessageContext messageContext){
        if(messageId<=0) return null;
        Bundle result = null;
        String msgHandlerID = mMessageHandlerMap.get(messageId);
        if(TextUtils.isEmpty(msgHandlerID)) {
            statMessageNotFindException(messageId);
            String msg = "sendMessageSync can not find event GundamCommander for message,id:"+ getLogMessageId(messageId) + " execute thread:" + (isMainThread()?"main thread":"background thread");
            FrameworkService.addDebugLog(TAG, ILogHandler.LogLevel.error,msg);
            return  result;
        }
        GundamMessage gundamMessage = mMessageDefinitionMap.get(messageId);
        if(gundamMessage==null){
            statMessageNotFindException(messageId);
            String msg = "sendMessageSync can not find event definition for "+ msgHandlerID +" message,id:"+ getLogMessageId(messageId) + " in GundamCommander:" + msgHandlerID + " execute thread:" + (isMainThread()?"main thread":"background thread");
            FrameworkService.addDebugLog(TAG, ILogHandler.LogLevel.error,msg);
            return result;
        }

        if(FrameworkService.getInstance().mIsDebug){
            String msg = "sendMessageSync message to "+ msgHandlerID +",id:"+ getLogMessageId(messageId);
            if(messageData!=null){
                StringBuffer stringBuffer = new StringBuffer();
                final Set<String> keySet = messageData.keySet();
                for (final String key: keySet) {
                    stringBuffer.append('\"');
                    stringBuffer.append(key);
                    stringBuffer.append("\"=\"");
                    stringBuffer.append(messageData.get(key));
                    stringBuffer.append("\", ");
                }
                msg = msg + " data:"+ stringBuffer.toString();
            }else{
                msg = msg  + " without data";
            }
            msg += ",thread_mode:" + gundamMessage.threadMode +  ",execute thread: " + (isMainThread()?"main thread":"background thread");
            FrameworkService.addDebugLog(TAG, ILogHandler.LogLevel.info,msg);
        }

        //同步消息直接加载
        IModuleEntry moduleEntry = mControllerCenter.getModuleEntryBy(msgHandlerID);
        if(!moduleEntry.isLoaded() && gundamMessage!=null){
            HashMap<String, String> data = new HashMap<String, String>();
            data.put(StatManager.Constant.KEY_LOAD_MODULE_BY, StatManager.Constant.VALUE_LOAD_MODULE_BY_SYS_MSG);
            data.put(StatManager.Constant.KEY_MSG_ID, String.valueOf(messageId));
            data.put(StatManager.Constant.KEY_MSG_NAME, gundamMessage.name);
            data.put(StatManager.Constant.KEY_MSG_THREAD_MODE,isMainThread()? "main":gundamMessage.threadMode);
            if(moduleEntry.isNotLoad()){
                boolean success = moduleEntry.loadModuleSyn();
                data.put(StatManager.Constant.KEY_MODULE_STATE, StatManager.Constant.VALUE_MODULE_STATE_NOT_LOAD);
                data.put(StatManager.Constant.KEY_MODULE_LOAD_FINISH, String.valueOf(success));
                FrameworkService.addStat(StatManager.Constant.CATEGORY_PERFORMANCE, data);

                if(success){
                    //防止出现异步加载消息发出之前（有缓存），但是同步加载先执行时，也要执行缓存的东西
                    moduleEntry.triggerCache();
                }else{
                    FrameworkService.addDebugLog(TAG, ILogHandler.LogLevel.error,"handleMessageSync failed loader module:"+ moduleEntry.getModuleInfo()!=null?moduleEntry.getModuleInfo().getID():"" +  " execute thread:" + (isMainThread()?"main thread":"background thread,on bundle "+moduleEntry.getModuleInfo().getID()));
                    //加载失败时，返回
                    return Bundle.EMPTY;
                }

            }else{
                //正在加载时，阻塞等待4s
                boolean isLoadFinish = waitTillLoadFinish(moduleEntry);
                data.put(StatManager.Constant.KEY_MODULE_STATE, StatManager.Constant.VALUE_MODULE_STATE_LOADING);
                data.put(StatManager.Constant.KEY_MODULE_LOAD_FINISH, String.valueOf(isLoadFinish));
                FrameworkService.addStat(StatManager.Constant.CATEGORY_PERFORMANCE, data);
                if(!isLoadFinish){
                    FrameworkService.addDebugLog(TAG, ILogHandler.LogLevel.error,"handleMessageSync failed get module "+ moduleEntry.getModuleInfo()!=null?moduleEntry.getModuleInfo().getID():"" + " execute thread:" + (isMainThread()?"main thread":"background thread,on bundle "+moduleEntry.getModuleInfo().getID()));
                    return  Bundle.EMPTY;
                }
            }
        }
        //如果RegisterAnnoation绑定在方法上面的,
        if(TextUtils.isEmpty(gundamMessage.method)){
            IMessageHandler handler = mControllerCenter.getMessageHandler(msgHandlerID);
            if (handler != null) {
                try{
                    long startTime = SystemClock.elapsedRealtime();
                    GMessage dispatchGMessage = GMessage.obtain(Integer.valueOf(messageId).intValue(),messageData);
                    dispatchGMessage.name = gundamMessage.name;
                    dispatchGMessage.messageContext = messageContext;
                    result =  handler.handleMessageSync(dispatchGMessage);
                    long timeInterval = SystemClock.elapsedRealtime() - startTime;
                    if (timeInterval > StatManager.ANR_INTERVAL){
                        HashMap<String, String> data = new HashMap<String, String>();
                        data.put(StatManager.Constant.KEY_JANK_TYPE, StatManager.Constant.KEY_MSG_JANK);
                        data.put(StatManager.Constant.KEY_MSG_ID, gundamMessage.name);
                        data.put(StatManager.Constant.KEY_MSG_THREAD_MODE,isMainThread()? "main":gundamMessage.threadMode);
                        data.put(StatManager.Constant.KEY_MSG_CLASS, handler.getClass().getSimpleName());
                        data.put(StatManager.Constant.KEY_JANK_INTERVAL, String.valueOf(timeInterval));
                        FrameworkService.addDebugLog(TAG, ILogHandler.LogLevel.info,"sendMessageSync处理超过2秒 " + data.toString());
                        FrameworkService.addStat(StatManager.Constant.CATEGORY_PERFORMANCE, data);
                    }
                }catch (Exception ex){
                    ex.printStackTrace();
                    statMessageException(gundamMessage,handler.getClass().getSimpleName(),ex.getMessage());
                    String messageLog = "handleMessageSync execute exception when dispatcher message, id:"+ getLogMessageId(gundamMessage.id)+",message name:" + gundamMessage.name+ ",target:" + gundamMessage.method+ " on thread " + gundamMessage.threadMode + " execute thread:" + (isMainThread()?"main thread":"background thread,on bundle "+moduleEntry.getModuleInfo().getID());
                    FrameworkService.addDebugLog(TAG, messageLog, ex);
                    throw ex;
                }
            }else {
                statMessageException(gundamMessage,handler.getClass().getSimpleName(),"");
                String msg = "handleMessageSync can not find event executor "+ msgHandlerID +" for message,id:"+ getLogMessageId(gundamMessage.id)+",message name:" + gundamMessage.name+ ",target:handleMessageSync on thread " + gundamMessage.threadMode + " execute thread:" + (isMainThread()?"main thread":"background thread,on bundle "+moduleEntry.getModuleInfo().getID());
                FrameworkService.addDebugLog(TAG, ILogHandler.LogLevel.error,msg);
                return result;
            }
        }else{
            long startTime = SystemClock.elapsedRealtime();
            Object handler = null;
            try{
                handler = mControllerCenter.getMessageHandlerObject(msgHandlerID);
                if(handler==null){
                    statMessageException(gundamMessage,"","");
                    String msg = "handleMessageSync can not find event executor "+ msgHandlerID +" for message id:"+ getLogMessageId(gundamMessage.id)+",message name:" + gundamMessage.name+ ",target:handleMessageSync on thread " + gundamMessage.threadMode + " execute thread:" + (isMainThread()?"main thread":"background thread,on bundle "+moduleEntry.getModuleInfo().getID());
                    FrameworkService.addDebugLog(TAG, ILogHandler.LogLevel.error,msg);
                    return result;
                }
                Method method = null;
                if(mMessageMethodMap.containsKey(gundamMessage.id)){
                    method = mMessageMethodMap.get(gundamMessage.id);
                }else{
                    Class ownerClass = handler.getClass();
                    Class[] argsClass = new Class[]{GMessage.class};
                    method = ownerClass.getDeclaredMethod(gundamMessage.method,argsClass);
                    method.setAccessible(true);
                    mMessageMethodMap.put(gundamMessage.id,method);
                }
                GMessage dispatchGMessage = GMessage.obtain(Integer.valueOf(messageId).intValue(),messageData);
                dispatchGMessage.name = gundamMessage.name;
                dispatchGMessage.messageContext = messageContext;
                if(method.getGenericReturnType().toString().equals("class android.os.Bundle")){
                    result  = (Bundle)method.invoke(handler, new Object[]{dispatchGMessage});
                }else{
                    method.invoke(handler, new Object[]{dispatchGMessage});
                }
                if(FrameworkService.getInstance().mIsDebug){
                    String logMessage = "handleMessageSync success to dispatcher "+ msgHandlerID +" message, id:"+ getLogMessageId(gundamMessage.id)+",message name:" + gundamMessage.name+ ",target:" + gundamMessage.method+ " on thread " + gundamMessage.threadMode + " execute thread:" + (isMainThread()?"main thread":"background thread,on bundle "+moduleEntry.getModuleInfo().getID());
                    FrameworkService.addDebugLog(TAG, ILogHandler.LogLevel.info,logMessage);
                }
                //反射调用 handler的Method方法
                long timeInterval = SystemClock.elapsedRealtime() - startTime;
                if (timeInterval > StatManager.ANR_INTERVAL){
                    HashMap<String, String> data = new HashMap<String, String>();
                    data.put(StatManager.Constant.KEY_JANK_TYPE, StatManager.Constant.KEY_MSG_JANK);
                    data.put(StatManager.Constant.KEY_MSG_ID, gundamMessage.name);
                    data.put(StatManager.Constant.KEY_MSG_THREAD_MODE,isMainThread()? "main":gundamMessage.threadMode);
                    data.put(StatManager.Constant.KEY_MSG_CLASS, handler.getClass().getSimpleName());
                    data.put(StatManager.Constant.KEY_JANK_INTERVAL, String.valueOf(timeInterval));
                    FrameworkService.addDebugLog(TAG, ILogHandler.LogLevel.info,"sendMessageSync处理超过2秒 " + data.toString());
                    FrameworkService.addStat(StatManager.Constant.CATEGORY_PERFORMANCE, data);
                }
            } catch (IllegalAccessException ex) {
                ex.printStackTrace();
                statMessageException(gundamMessage,handler.getClass().getSimpleName(),ex.getMessage());
                String messageLog = "handleMessageSync execute exception when dispatcher message, id:"+ getLogMessageId(gundamMessage.id)+",message name:" + gundamMessage.name+ ",target:" + gundamMessage.method+ " on thread " + gundamMessage.threadMode + " execute thread:" + (isMainThread()?"main thread":"background thread,on bundle "+moduleEntry.getModuleInfo().getID());
                FrameworkService.addDebugLog(TAG, messageLog, ex);
            } catch (NoSuchMethodException ex) {
                ex.printStackTrace();
                statMessageException(gundamMessage,handler.getClass().getSimpleName(),ex.getMessage());
                String messageLog = "handleMessageSync execute exception when dispatcher message, id:"+ getLogMessageId(gundamMessage.id)+",message name:" + gundamMessage.name+ ",target:" + gundamMessage.method+ " on thread " + gundamMessage.threadMode + " execute thread:" + (isMainThread()?"main thread":"background thread,on bundle "+moduleEntry.getModuleInfo().getID());
                FrameworkService.addDebugLog(TAG, messageLog, ex);
            } catch (InvocationTargetException ex) {
                ex.printStackTrace();
                statMessageException(gundamMessage,handler.getClass().getSimpleName(),ex.getCause().getMessage());
                String messageLog = "handleMessageSync execute exception when dispatcher message, id:"+ getLogMessageId(gundamMessage.id)+",message name:" + gundamMessage.name+ ",target:" + gundamMessage.method+ " on thread " + gundamMessage.threadMode + " execute thread:" + (isMainThread()?"main thread":"background thread,on bundle "+moduleEntry.getModuleInfo().getID());
                FrameworkService.addDebugLog(TAG, messageLog, ex);
            }catch (Exception ex) {
                ex.printStackTrace();
                statMessageException(gundamMessage,handler.getClass().getSimpleName(),ex.getMessage());
                String messageLog = "handleMessageSync execute exception when dispatcher message, id:"+ getLogMessageId(gundamMessage.id)+",message name:" + gundamMessage.name+ ",target:" + gundamMessage.method+ " on thread " + gundamMessage.threadMode + " execute thread:" + (isMainThread()?"main thread":"background thread,on bundle "+moduleEntry.getModuleInfo().getID());
                FrameworkService.addDebugLog(TAG, messageLog, ex);
                throw ex;
            }
        }
        return result;
    }

    private void statMessageException(GundamMessage gundamMessage,String handlerName,String expMessage){
        HashMap<String, String> data = new HashMap<String, String>();
        data.put(StatManager.Constant.KEY_JANK_TYPE, StatManager.Constant.KEY_MSG_EXP);
        data.put(StatManager.Constant.KEY_MSG_ID, gundamMessage.name);
        data.put(StatManager.Constant.KEY_MSG_CLASS, handlerName);
        data.put(StatManager.Constant.KEY_MSG_DETAIL, expMessage);
        FrameworkService.addStat(StatManager.Constant.CATEGORY_EXCEPTION, data);
    }

    private void statMessageNotFindException(int messageId){
        HashMap<String, String> data = new HashMap<String, String>();
        data.put(StatManager.Constant.KEY_JANK_TYPE, StatManager.Constant.KEY_MSG_DEF_EXP);
        data.put(StatManager.Constant.KEY_MSG_ID, getLogMessageId(messageId));
        data.put(StatManager.Constant.KEY_MSG_NAME, getLogMessageId(messageId));
        FrameworkService.addStat(StatManager.Constant.CATEGORY_EXCEPTION, data);
    }
    private void statMessageNotFindException(String messageName){
        HashMap<String, String> data = new HashMap<String, String>();
        data.put(StatManager.Constant.KEY_JANK_TYPE, StatManager.Constant.KEY_MSG_DEF_EXP);
        data.put(StatManager.Constant.KEY_MSG_ID, "");
        data.put(StatManager.Constant.KEY_MSG_NAME, messageName);
        FrameworkService.addStat(StatManager.Constant.CATEGORY_EXCEPTION, data);
    }

    /**
     * 阻塞等待直到模块加载完成
     * @param moduleEntry
     * @return 是否等到了加载完成
     */
    private boolean waitTillLoadFinish(IModuleEntry moduleEntry) {

        //最大等待时间为10*400ms
        final int MAX_WAIT_ROUNDS = 10;
        //每轮等待间隔
        final long WAIT_ROUND_INTERVAL = 400;

        int waitRounds = 0;
        while (waitRounds < MAX_WAIT_ROUNDS){
            LOG.d("ModuleLoader", "wait rounds: "+ waitRounds);
            try {
                Thread.sleep(WAIT_ROUND_INTERVAL);
            } catch (InterruptedException e) {
                e.printStackTrace();
            }

            if(moduleEntry.isLoaded()){
                LOG.d("ModuleLoader", "wait break");
                break;
            }

            waitRounds++;
        }
        return moduleEntry.isLoaded();
    }

    @Override
    public boolean handleMessage(android.os.Message msg) {
        if (msg.what != MSG_DEFAULT_ID) {
            return  false;
        }
        MessageData msgObj = (MessageData)msg.obj;
        Bundle bundle = (Bundle)msgObj.getBundle();
        IMessageContext messageContext = msgObj.getMessageContext();
        int messageID = bundle.getInt(KEY_MESSAGE_ID);
        Bundle messageBody = bundle.getBundle(KEY_MESSAGE_BODY);
        IResultListener listener = bundle.getParcelable(KEY_MESSAGE_LISTENER);

        String msgHandlerID = mMessageHandlerMap.get(messageID);
        if(TextUtils.isEmpty(msgHandlerID)) {
            String logMessage = "handleMessage can not find event GundamCommander for message, id:"+ getLogMessageId(messageID) + " execute thread:" + (isMainThread()?"main thread":"background thread");
            FrameworkService.addDebugLog(TAG, ILogHandler.LogLevel.error,logMessage);
            return  false;
        }

        GundamMessage gundamMessage = mMessageDefinitionMap.get(messageID);
        if(gundamMessage==null){
            gundamMessage = new GundamMessage();
            String logMessage = "handleMessage can not find event definition for message "+ msgHandlerID +", id:"+ getLogMessageId(messageID) + " in GundamCommander:" + msgHandlerID + " execute thread:" + (isMainThread()?"main thread":"background thread");
            FrameworkService.addDebugLog(TAG, ILogHandler.LogLevel.info,logMessage);
        }
        IModuleEntry moduleEntry = mControllerCenter.getModuleEntryBy(msgHandlerID);
        if(!moduleEntry.isLoaded()) {
            MessageOperation operation = new MessageOperation(messageID, gundamMessage.name,messageBody, listener,messageContext);
            moduleEntry.cacheOperation(operation);
            if(moduleEntry.isNotLoad()) {
                moduleEntry.loadModuleAsyn();
            }
            return false;
        }

        if(TextUtils.isEmpty(gundamMessage.method)){
            IMessageHandler handler = mControllerCenter.getMessageHandler(msgHandlerID);
            if (handler != null) {
                try{
                    GMessage dispatcherGMessage = GMessage.obtain(Integer.valueOf(messageID).intValue(),messageBody,listener);
                    dispatcherGMessage.name = gundamMessage.name;
                    dispatcherGMessage.messageContext= messageContext;
                    handler.handleMessage(dispatcherGMessage);
                    String logMessage = "handleMessage success to dispatcher message to "+ msgHandlerID +",id:"+ getLogMessageId(gundamMessage.id)+",message name:" + gundamMessage.name+ ",target:" + gundamMessage.method+ " on thread " + gundamMessage.threadMode + " execute thread:" + (isMainThread()?"main thread":"background thread,on bundle "+moduleEntry.getModuleInfo().getID());
                    FrameworkService.addDebugLog(TAG, ILogHandler.LogLevel.info,logMessage);

                }catch (Exception ex){
                    ex.printStackTrace();
                    statMessageException(gundamMessage,handler.getClass().getSimpleName(),ex.getMessage());
                    String messageLog = "handleMessage execute exception when dispatcher message to "+ msgHandlerID +",id:"+ getLogMessageId(gundamMessage.id)+",message name:" + gundamMessage.name+ ",target:" + gundamMessage.method+ " on thread " + gundamMessage.threadMode + " execute thread:" + (isMainThread()?"main thread":"background thread,on bundle "+moduleEntry.getModuleInfo().getID());
                    FrameworkService.addDebugLog(TAG, messageLog, ex);
                    throw ex;
                }
            }else{
                statMessageException(gundamMessage,"","");
                String logMessage = "handleMessage can not find MessageHandler for message "+ msgHandlerID +" ,id:"+ getLogMessageId(messageID) + " in GundamCommander:" + msgHandlerID + " execute thread:" + (isMainThread()?"main thread":"background thread,on bundle "+moduleEntry.getModuleInfo().getID());
                FrameworkService.addDebugLog(TAG, ILogHandler.LogLevel.error,logMessage);
            }
        }else{
            //反射调用 handler的Method方法
            long startTime = SystemClock.elapsedRealtime();
            Object handler=null;
            try{
                handler = mControllerCenter.getMessageHandlerObject(msgHandlerID);
                if(handler==null){
                    statMessageException(gundamMessage,"","");
                    String logMessage = "handleMessage can not find MessageHandler for message,id:"+ getLogMessageId(messageID) + " in GundamCommander:" + msgHandlerID + " execute thread:" + (isMainThread()?"main thread":"background thread,on bundle "+moduleEntry.getModuleInfo().getID());
                    FrameworkService.addDebugLog(TAG, ILogHandler.LogLevel.error,logMessage);
                }
                Method method = null;
                if(mMessageMethodMap.containsKey(gundamMessage.id)){
                    method = mMessageMethodMap.get(gundamMessage.id);
                }else{
                    Class ownerClass = handler.getClass();
                    Class[] argsClass = new Class[]{GMessage.class};
                    method = ownerClass.getDeclaredMethod(gundamMessage.method,argsClass);
                    method.setAccessible(true);
                    mMessageMethodMap.put(gundamMessage.id,method);
                }
                GMessage dispatchGMessage = GMessage.obtain(Integer.valueOf(messageID).intValue(),messageBody,listener);
                dispatchGMessage.name = gundamMessage.name;
                dispatchGMessage.messageContext= messageContext;
                method.invoke(handler, new Object[]{dispatchGMessage});

                String logMessage = "handleMessage success to dispatcher message "+ msgHandlerID +",id:"+ getLogMessageId(gundamMessage.id)+",message name:" + gundamMessage.name+ ",target:" + gundamMessage.method+ " on thread " + gundamMessage.threadMode + " execute thread:" + (isMainThread()?"main thread":"background thread,on bundle "+moduleEntry.getModuleInfo().getID());
                FrameworkService.addDebugLog(TAG, ILogHandler.LogLevel.info,logMessage);

                //反射调用 handler的Method方法
                long timeInterval = SystemClock.elapsedRealtime() - startTime;
                if (timeInterval > StatManager.ANR_INTERVAL){
                    HashMap<String, String> data = new HashMap<String, String>();
                    data.put(StatManager.Constant.KEY_JANK_TYPE, StatManager.Constant.KEY_MSG_JANK);
                    data.put(StatManager.Constant.KEY_MSG_ID, gundamMessage.name);
                    data.put(StatManager.Constant.KEY_MSG_THREAD_MODE,isMainThread()? "main":gundamMessage.threadMode);
                    data.put(StatManager.Constant.KEY_MSG_CLASS, handler.getClass().getSimpleName());
                    data.put(StatManager.Constant.KEY_JANK_INTERVAL, String.valueOf(timeInterval));
                    FrameworkService.addDebugLog(TAG, ILogHandler.LogLevel.info,"handleMessage处理超过2秒 " + data.toString());
                    FrameworkService.addStat(StatManager.Constant.CATEGORY_PERFORMANCE, data);
                }
            }catch (IllegalAccessException ex) {
                ex.printStackTrace();
                statMessageException(gundamMessage,handler.getClass().getSimpleName(),ex.getMessage());
                String messageLog = "handleMessage execute exception when dispatcher message,id:"+ getLogMessageId(gundamMessage.id)+",message name:" + gundamMessage.name+ ",target:" + gundamMessage.method+ " on thread " + gundamMessage.threadMode + " execute thread:" + (isMainThread()?"main thread":"background thread,on bundle "+moduleEntry.getModuleInfo().getID());
                FrameworkService.addDebugLog(TAG, messageLog, ex);
            } catch (NoSuchMethodException ex) {
                ex.printStackTrace();
                statMessageException(gundamMessage,handler.getClass().getSimpleName(),ex.getMessage());
                String messageLog = "handleMessage execute exception when dispatcher message,id:"+ getLogMessageId(gundamMessage.id)+",message name:" + gundamMessage.name+ ",target:" + gundamMessage.method+ " on thread " + gundamMessage.threadMode + " execute thread:" + (isMainThread()?"main thread":"background thread,on bundle "+moduleEntry.getModuleInfo().getID());
                FrameworkService.addDebugLog(TAG, messageLog, ex);
            } catch (InvocationTargetException ex) {
                statMessageException(gundamMessage,handler.getClass().getSimpleName(),ex.getCause().getMessage());
                String messageLog = "handleMessage execute exception when dispatcher message,id:"+ getLogMessageId(gundamMessage.id)+",message name:" + gundamMessage.name+ ",target:" + gundamMessage.method+ " on thread " + gundamMessage.threadMode + " execute thread:" + (isMainThread()?"main thread":"background thread,on bundle "+moduleEntry.getModuleInfo().getID());
                FrameworkService.addDebugLog(TAG, messageLog, ex);
            }catch (Exception ex){
                statMessageException(gundamMessage,handler.getClass().getSimpleName(),ex.getMessage());
                String messageLog = "handleMessage execute exception when dispatcher message,id:"+ getLogMessageId(gundamMessage.id)+",message name:" + gundamMessage.name+ ",target:" + gundamMessage.method+ " on thread " + gundamMessage.threadMode + " execute thread:" + (isMainThread()?"main thread":"background thread,on bundle "+moduleEntry.getModuleInfo().getID());
                FrameworkService.addDebugLog(TAG, messageLog, ex);
                throw ex;
            }
        }
        return true;
    }

    void invokeSubscriber(PendingPost pendingPost) {
        GundamMessage event = pendingPost.event;
        Message subscription = pendingPost.subscription;
        PendingPost.releasePendingPost(pendingPost);
        invokeSubscriber(subscription,event);
    }

    void invokeSubscriber(Message subscription,GundamMessage event) {
        handleMessage(subscription);
    }

    private boolean isMainThread(){
        return Thread.currentThread().getId() == Looper.getMainLooper().getThread().getId();
    }

    private String getLogMessageId(int messageId){
        return String.valueOf(messageId) + "(" + String.format("0x%08x", messageId) +")";
    }


}
