package cn.ninegame.genericframework.hybrid;

import android.text.TextUtils;
import android.util.Log;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import cn.ninegame.genericframework.basic.GMessage;
import cn.ninegame.genericframework.basic.GundamCommander;
import cn.ninegame.genericframework.basic.IResultListener;
import cn.ninegame.genericframework.basic.Notification;
import cn.ninegame.genericframework.basic.RegisterMessage;
import cn.ninegame.genericframework.basic.RegisterMessages;

/**
 * <p>Title: ucweb</p>
 * <p/>
 * <p>Description: </p>
 * description
 * <p>Copyright: Copyright (c) 2016</p>
 * <p/>
 * <p>Company: ucweb.com</p>
 *
 * @author ZhuZhiFan (zhifan.zzf@alibaba-inc.com)
 * @date 2016/11/25
 * changed by Zhonghui
 * @date 2018/03/10
 */
@RegisterMessages({
        @RegisterMessage(name=NativeAppEventController.MSG_REGISTER_EVENT),
        @RegisterMessage(name=NativeAppEventController.MSG_UNREGISTER_EVENT),
                @RegisterMessage(name=NativeAppEventController.MSG_TRIGGER_EVENT),
                        @RegisterMessage(name=NativeAppEventController.MSG_CLEAN_EVENT),
})
public class NativeAppEventController extends GundamCommander {

    public static final String MSG_REGISTER_EVENT = "msg_common_register_event";
    public static final String MSG_UNREGISTER_EVENT = "msg_common_cancel_event";
    public static final String MSG_TRIGGER_EVENT = "msg_common_trigger_event";
    public static final String MSG_CLEAN_EVENT = "msg_common_clean_event";
    public static final String KEY_EVENT_ID = "type";

    private Map<String, ArrayList<IResultListener>> mListenerMap = new HashMap<String, ArrayList<IResultListener>>();
    private List<String> mRegisteredEventList = new LinkedList<String>();

    @Override
    public void handleMessage(GMessage gMessage) {
        Map<String, String> argMap = null;
        if (gMessage.bundleData != null) {
            try {
                argMap = (HashMap<String, String>) gMessage.bundleData.getSerializable(NativeApp.ARG_ARGS);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        String eventId = null;
        if (argMap != null) {
            eventId = argMap.get(KEY_EVENT_ID);
        }
        ArrayList<IResultListener> listenerList = mListenerMap.get(eventId);

        if (gMessage.resultListener != null) {
            if (MSG_CLEAN_EVENT.equals(gMessage.name)) {
                // 清除所有注册的消息
                for (String event : mListenerMap.keySet()) {
                    unregisterNotification(eventId, this);
                }
                mListenerMap.clear();
                mRegisteredEventList.clear();
            } else if (!TextUtils.isEmpty(eventId)) {
                if (MSG_REGISTER_EVENT.equals(gMessage.name)) {
                    //register notification
                    Log.d("NativeOnEvent", "注册监听 eventId = " + eventId);

                    if (listenerList == null) {
                        listenerList = new ArrayList<IResultListener>();
                        mListenerMap.put(eventId, listenerList);
                    }
                    if (!listenerList.contains(gMessage.resultListener)) {
                        listenerList.add(gMessage.resultListener);
                    }

                    if (mRegisteredEventList.contains(eventId)) {
                        //消息已经注册监听
                        return;
                    }
                    registerNotification(eventId, this);
                    mRegisteredEventList.add(eventId);

                } else if (MSG_UNREGISTER_EVENT.equals(gMessage.name)) {
                    // unregister notification
                    Log.d("NativeOnEvent", "反注册监听 eventId = " + eventId);
                    if (listenerList != null) {
                        listenerList.remove(gMessage.resultListener);
                        if (listenerList.isEmpty()) {
                            mRegisteredEventList.remove(eventId);
                            unregisterNotification(eventId, this);
                        }
                    }

                } else if (MSG_TRIGGER_EVENT.equals(gMessage.name)) {
                    sendNotification(eventId, gMessage.bundleData);
                }
            }
        }
    }

    @Override
    public void onNotify(Notification notification) {
        super.onNotify(notification);
        ArrayList<IResultListener> listenerList = mListenerMap.get(notification.messageName);
        if (listenerList != null) {
            for (IResultListener listener : listenerList) {
                notification.bundleData.putString(KEY_EVENT_ID, notification.messageName);
                listener.onResult(notification.bundleData);
            }
        }
    }
}
