package cn.ninegame.genericframework.module;

import android.content.Context;
import android.os.Handler;
import android.os.Looper;
import android.os.Message;

import java.io.File;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import cn.ninegame.genericframework.basic.FrameworkFacade;
import cn.ninegame.genericframework.config.ModulePathManager;
import cn.ninegame.genericframework.module.CacheOperation.DialogOperation;
import cn.ninegame.genericframework.module.CacheOperation.FragmentOperation;
import cn.ninegame.genericframework.module.CacheOperation.LoadFragmentOperation;
import cn.ninegame.genericframework.module.CacheOperation.MessageOperation;
import cn.ninegame.genericframework.module.CacheOperation.MultiFragmentOperation;
import cn.ninegame.genericframework.module.CacheOperation.Operation;
import cn.ninegame.genericframework.tools.LOG;

/**
 * ****************************************************************************
 * Copyright @ 2009 - 2015 www.9game.cn All Rights Reserved
 * <p/>
 * Creation    : 2016-01-23
 * Author      : lihq@ucweb.com
 * Description : Tell me what does this class do
 * ****************************************************************************
 */
abstract class AbsModuleEntry implements IModuleEntry{

    protected ModuleInfo mModuleInfo;

    public final static int LOAD_STATE_NOT_LOAD = 0;
    public final static int LOAD_STATE_LOADING = 1;
    public final static int LOAD_STATE_LOADED = 2;

    volatile int mLoadState = LOAD_STATE_NOT_LOAD;

    protected Context mBaseContext;

    protected String mApplicationID;

    protected ModuleApplication mApplication = null;

    private ArrayList<ILoadModuleListener> mListeners = new ArrayList<ILoadModuleListener>();

    private List<Operation> mOperationCache = Collections.synchronizedList(new ArrayList<Operation>());

    private Handler mModuleHandler;

    public void setModuleHandler(Handler handler){
        mModuleHandler = handler;
    }

    public void loadModuleAsyn(){
        if(mModuleHandler != null) {
            LOG.d("ModuleLoader","load module asyn");
            if (isNotLoad()) {
                setLoadState(AbsModuleEntry.LOAD_STATE_LOADING);

                Message message = Message.obtain();
                message.what = ModuleCenter.MSG_LOAD_MODULE;
                message.obj = mModuleInfo;
                mModuleHandler.sendMessage(message);
            }
        }
    }

    public boolean loadModuleSyn(){
        LOG.d("ModuleLoader", "load module syn");
        boolean bResult = false;
        if (isLoaded()) {
            bResult = true;
        }
        if (isNotLoad()) {
            setLoadState(AbsModuleEntry.LOAD_STATE_LOADING);
            bResult = loadModule();
        }
        return bResult;
    }

    protected boolean loadModule(){
        return false;
    }

    private Operation getFirstCacheOperation(){
        if(mOperationCache.size() > 0){
            return mOperationCache.get(0);
        }

        return null;
    }

    public void cacheOperation(Operation op){
        LOG.d("ModuleLoader", "cache operation:" + op.getOperationKey());
        if(op instanceof FragmentOperation){
            for (Operation opera : mOperationCache){
                if(opera != null && opera.getOperationKey()==op.getOperationKey()){
                    return;
                }
            }
        }
        mOperationCache.add(op);
    }

    /**
     * 执行缓存里面的所有操作（理想：fragment相关只执行一次，message可以执行多个）
     */
    public void triggerCache(){
        for(Operation op : mOperationCache){
            LOG.d("ModuleLoader", "trigger operation:" + op.getOperationKey() + ", Main Loop = " + (Looper.myLooper() == Looper.getMainLooper()));

            if(op instanceof MessageOperation){
                MessageOperation mo = (MessageOperation) op;
                FrameworkFacade.getInstance().getEnvironment().sendMessageForResult(mo.messageID, mo.messageData, mo.listener);
            }else if(op instanceof FragmentOperation){
                FragmentOperation fo = (FragmentOperation)op;
                switch (fo.type){
                    case FragmentOperation.OPERATION_TYPE_FRAGMENT_WITH_FLAG:
                        FrameworkFacade.getInstance().getEnvironment().startFragmentWithFlagNoIntercept(fo.fragmentID,fo.param,fo.useAnim,fo.flag);
                        break;
                    case FragmentOperation.OPERATION_TYPE_FRAGMENT_WITH_MODE:
                        FrameworkFacade.getInstance().getEnvironment().startFragmentForResultNoIntercept(fo.fragmentID, fo.param, fo.listener,fo.useAnim,fo.mode);
                        break;
                }
            }else if(op instanceof MultiFragmentOperation){
                MultiFragmentOperation mfo = (MultiFragmentOperation)op;
                FrameworkFacade.getInstance().getEnvironment().startFragmentsNoIntercept(mfo.fragmentIDs, mfo.params, mfo.modes);
            }else if (op instanceof DialogOperation) {
                DialogOperation dlgOp = (DialogOperation)op;
                FrameworkFacade.getInstance().getEnvironment().startDialogFragmentNoIntercept(dlgOp.fragmentID, dlgOp.param, dlgOp.listener);
            }else if (op instanceof LoadFragmentOperation) {
                LoadFragmentOperation lfo = (LoadFragmentOperation)op;
                FrameworkFacade.getInstance().getEnvironment().loadFragment(lfo.getFragmentID(),lfo.getListener());
            }
        }

        //执行完就清除
        mOperationCache.clear();
    }

    public void addLoadModuleListener(ILoadModuleListener listener){
        mListeners.add(listener);
    }

    /**
     * notify the observers before
     */
    public void notifyOnLoadStart(){
        setLoadState(LOAD_STATE_LOADING);

        for (ILoadModuleListener listener: mListeners){
            listener.onLoadStart(mModuleInfo, getFirstCacheOperation());
        }
    }

    public void notifyOnLoadFinish(){
        setLoadState(LOAD_STATE_LOADED);
        if(mListeners!=null){
            for (ILoadModuleListener listener: mListeners){
                listener.onLoadFinish(mModuleInfo, getFirstCacheOperation());
            }
        }
    }

    public void notifyOnLoadError(){
        setLoadState(LOAD_STATE_NOT_LOAD);
        if(mListeners!=null){
            for(ILoadModuleListener listener : mListeners){
                listener.onLoadError(mModuleInfo, getFirstCacheOperation());
            }
        }
    }

    public String getModuleRootPath(){
        if(mModuleInfo.isAndroidLibrary){
            return mApplication.getContext().getApplicationInfo().dataDir;
        }else{
            return ModulePathManager.getInstance().getModuleRootDirPath(mModuleInfo.id);
        }
    }

    public String getModuleDataPath(){
        if(mModuleInfo.isAndroidLibrary){
            return mApplication.getContext().getApplicationInfo().dataDir;
        }else{
            return ModulePathManager.getInstance().getModuleDataDirPath(mModuleInfo.id);
        }

    }

    public String getModuleSoPath(){
        if(mModuleInfo.isAndroidLibrary){
            return mApplication.getContext().getApplicationInfo().dataDir + File.pathSeparator;
        }else{
            return ModulePathManager.getInstance().getModuleSODirPath(mModuleInfo.id,mModuleInfo.version);
        }
    }

    public boolean isLoaded() {
        return mLoadState == LOAD_STATE_LOADED;
    }

    public int getLoadState(){
        return mLoadState;
    }

    public void setLoadState(int state) {
        this.mLoadState = state;
    }

    public ModuleInfo getModuleInfo() {
        return mModuleInfo;
    }

    public void setModuleInfo(ModuleInfo moduleInfo) {
        this.mModuleInfo = moduleInfo;
    }

    @Override
    public void setBaseContext(Context base) {
        mBaseContext = base;
    }

    @Override
    public void setApplicationID(String applicationID) {
        mApplicationID = applicationID;
    }

    @Override
    public void destroy() {
        if (mApplication != null) {
            mApplication.onDestroy();
        }
    }
}
