package cn.ninegame.genericframework.ui;

import android.app.Activity;
import android.content.Intent;
import android.os.Bundle;
import android.support.v4.app.Fragment;
import android.text.TextUtils;

import java.util.HashMap;

import cn.ninegame.genericframework.basic.Environment;
import cn.ninegame.genericframework.basic.FrameworkService;
import cn.ninegame.genericframework.basic.GundamFragment;
import cn.ninegame.genericframework.basic.ILoadFragmentListener;
import cn.ninegame.genericframework.basic.IResultListener;
import cn.ninegame.genericframework.basic.StatManager;
import cn.ninegame.genericframework.module.CacheOperation.DialogOperation;
import cn.ninegame.genericframework.module.CacheOperation.FragmentOperation;
import cn.ninegame.genericframework.module.CacheOperation.LoadFragmentOperation;
import cn.ninegame.genericframework.module.CacheOperation.MultiFragmentOperation;
import cn.ninegame.genericframework.module.IModuleComponent;
import cn.ninegame.genericframework.module.IModuleEntry;
import cn.ninegame.genericframework.module.ModuleCenter;

/**
 * Created by lingcong on 2015/6/2.
 */
public class FragmentCenter {
    private ModuleCenter mModuleCenter ;
    private Environment mEnv;

    private HashMap<String,String> mFragmentModuleIDMap = new HashMap<String, String>(2);
    private HashMap<String,String> mFragmentAliasMap = new HashMap<String, String>(2);

    private static FragmentCenter sInstance;

    public void setModuleCenter(ModuleCenter center) {
        mModuleCenter = center;
    }

    public void setEnv(Environment env) {
        mEnv = env;
    }

    public FragmentCenter() {
        sInstance = this;
    }


    public void startFragmentForResult(String fragmentID, Bundle param, IResultListener listener, boolean useAnim, int mode) {
        String moduleID = getHostModuleID(fragmentID);
        IModuleEntry moduleEntry = getModuleEntry(moduleID);

        if(!moduleEntry.isLoaded()){
            FragmentOperation op = new FragmentOperation(fragmentID,param,listener,useAnim,mode);
            moduleEntry.cacheOperation(op);

            //正在加载不需要发送
            loadModuleAsynIfNeed(moduleEntry);
            return;
        }

        BaseFragment fragment = moduleEntry.loadFragment(fragmentID);
        if (fragment == null) {
            return;
        }
        fragment.setEnvironment(mEnv);
        if(param != null) {
            fragment.setBundleArguments(param);
        }
        fragment.setResultListener(listener);
        fragment.setUseAnim(useAnim);
        fragment.hideKeyboard();

        pushFragment(mEnv.getCurrentActivity() , new BaseFragment[]{fragment}, new int[]{mode});
    }

    /**
     * 只支持所有Fragment是同一个Module和同一个Activity的场景
     * @param fragmentIDs
     * @param params
     */
    public void startFragments(String[] fragmentIDs, Bundle[] params, int[] modes){
        if(fragmentIDs.length != params.length || fragmentIDs.length != modes.length){
            new Throwable("fragment and bundle have different length!");
        }

        String moduleID = getHostModuleID(fragmentIDs[0]);
        IModuleEntry moduleEntry = getModuleEntry(moduleID);

        if(!moduleEntry.isLoaded()){
            MultiFragmentOperation op = new MultiFragmentOperation(fragmentIDs, params, modes);
            moduleEntry.cacheOperation(op);

            loadModuleAsynIfNeed(moduleEntry);
            return;
        }

        BaseFragment[] baseFragments = new BaseFragment[fragmentIDs.length];

        for(int i = 0; i < fragmentIDs.length; i++){
            BaseFragment fragment = moduleEntry.loadFragment(fragmentIDs[i]);
            if (fragment == null) {
                continue;
            }
            baseFragments[i] = fragment;
            fragment.setEnvironment(mEnv);
            if(params[i] != null) {
                fragment.setBundleArguments(params[i]);
            }
            fragment.setUseAnim(false);
            fragment.hideKeyboard();
        }

        pushFragment(mEnv.getCurrentActivity(), baseFragments, modes);
    }

    private void loadModuleAsynIfNeed(final IModuleEntry moduleEntry) {
        //正在加载不需要发送
        if(moduleEntry.isNotLoad()){
            moduleEntry.loadModuleAsyn();
        }
    }

    public void startDialogFragment(String fragmentID, Bundle param, IResultListener listener){

        if(!(mEnv.getCurrentActivity() instanceof BaseActivity)) {
            // 容错处理
            return;
        }
        String moduleID = getHostModuleID(fragmentID);
        IModuleEntry moduleEntry = getModuleEntry(moduleID);

        if(!moduleEntry.isLoaded()){
            DialogOperation op = new DialogOperation(fragmentID, param, listener);
            moduleEntry.cacheOperation(op);

            loadModuleAsynIfNeed(moduleEntry);
            return;
        }

        BaseDialogFragment dialogFragment = moduleEntry.loadDialogFragment(fragmentID);
        dialogFragment.setEnvironment(mEnv);
        if(param != null) {
            dialogFragment.setBundleArguments(param);
        }
        dialogFragment.setResultListener(listener);

        ((BaseActivity)mEnv.getCurrentActivity()).showDialogFragment(dialogFragment);
    }

    public void startFragmentWithFlag(String fragmentID, Bundle param, boolean useAnim, int intentFlag){

        String moduleID = getHostModuleID(fragmentID);
        IModuleEntry moduleEntry = getModuleEntry(moduleID);

        if(!moduleEntry.isLoaded()){
            FragmentOperation op = new FragmentOperation(fragmentID, param, useAnim,intentFlag);
            moduleEntry.cacheOperation(op);

            loadModuleAsynIfNeed(moduleEntry);
            return;
        }

        BaseFragment fragment = moduleEntry.loadFragment(fragmentID);
        fragment.setEnvironment(mEnv);
        if(param != null) {
            fragment.setBundleArguments(param);
        }
        fragment.setUseAnim(useAnim);
        fragment.hideKeyboard();

        pushFragmentWithFlag(mEnv.getCurrentActivity(), fragment, intentFlag);
    }

    private String getHostModuleID(String fragmentID) {
        String moduleID = mFragmentModuleIDMap.get(fragmentID);
        return moduleID;
    }

    private IModuleEntry getModuleEntry(String moduleID) {
        IModuleEntry moduleEntry = mModuleCenter.getModuleEntry(moduleID);
        if(moduleEntry == null) {
            moduleEntry = mModuleCenter.getDefaultModuleEntry();
        }

        return moduleEntry;
    }

    public void registerModuleFragment(String ModuleID,GundamFragment[] fragmentIDs) {
        if(fragmentIDs == null || fragmentIDs.length == 0) {
            return;
        }

        for (int i = 0;i<fragmentIDs.length;i++) {
            mFragmentModuleIDMap.put(fragmentIDs[i].className,ModuleID);
            mFragmentAliasMap.put(fragmentIDs[i].name,fragmentIDs[i].className);
        }
    }

    public void reInitFragment(Fragment fragment) {
        if (fragment instanceof IModuleComponent) {
            IModuleComponent moduleComponent = (IModuleComponent)fragment;
            moduleComponent.setEnvironment(mEnv);

            String moduleID = getHostModuleID(moduleComponent.getClass().getName());
            IModuleEntry moduleEntry = getModuleEntry(moduleID);
            moduleComponent.setModuleEntry(moduleEntry);
        }
    }

    public void loadFragment(String fragmentID,ILoadFragmentListener listener) {
        if (TextUtils.isEmpty(fragmentID) || (listener == null)) {
            return;
        }

        String moduleID = getHostModuleID(fragmentID);
        if (TextUtils.isEmpty(moduleID)) {
            listener.onFailed();
            return;
        }

        IModuleEntry moduleEntry = getModuleEntry(moduleID);

        if(!moduleEntry.isLoaded()){
            LoadFragmentOperation lfo = new LoadFragmentOperation(fragmentID,listener);
            moduleEntry.cacheOperation(lfo);
            //正在加载不需要发送
            loadModuleAsynIfNeed(moduleEntry);
            return;
        }

        BaseFragment fragment = moduleEntry.loadFragment(fragmentID);
        if (fragment == null) {
            listener.onFailed();
            return;
        }

        fragment.setEnvironment(mEnv);

        if (listener != null) {
            listener.onSucess(fragment);
        }

    }

    private static HashMap<String, BaseFragment> gSwitchCache = new HashMap<String, BaseFragment>(2);

    private void pushFragment(Activity baseActivity, BaseFragment[] fragmentList, int[] modes) {

        Class hostActivity = fragmentList[0].getHostActivity();

        if (baseActivity != null) {
            if (!baseActivity.isFinishing()
                    && baseActivity.getClass() == hostActivity
                    && baseActivity instanceof BaseActivity
                    && ((BaseActivity) baseActivity).isPerformResumed()) {
                //current Activity is hostActivity
                for (int i=0; i<fragmentList.length; ++i){
                    ((BaseActivity) baseActivity).pushFragment(fragmentList[i], modes[i]);
                }

            } else if (hostActivity != null) {
                //cache Fragment
                String[] fragmentNames = new String[fragmentList.length];

                for (int i = 0; i < fragmentList.length; i++ ){
                    gSwitchCache.put(fragmentList[i].getName(), fragmentList[i]);
                    fragmentNames[i] = fragmentList[i].getName();
                    fragmentList[i].setUserActivityAnim(true);
                }

                Intent intent = new Intent(Intent.ACTION_VIEW);
                intent.setClass(baseActivity, hostActivity);
                intent.putExtra(BaseActivity.INTENT_EXTRA_FRAGMENT_TAG, fragmentNames);
                intent.putExtra(BaseActivity.INTENT_EXTRA_LAUNCHER_MODE, modes);

                baseActivity.startActivity(intent);

                //use Anim only start another activity
                if (fragmentList[fragmentList.length - 1].isUseAnim()) {
                    baseActivity.overridePendingTransition(fragmentList[fragmentList.length - 1].mEnterAnimRes, fragmentList[fragmentList.length - 1].mExitAnimRes);
                } else {
                    baseActivity.overridePendingTransition(0, 0);
                }

            } else if (baseActivity instanceof BaseActivity) {
                if (baseActivity.isFinishing()) {
                    StringBuffer fragmentNameBuffer = new StringBuffer();
                    for (BaseFragment fragment : fragmentList){
                        fragmentNameBuffer.append(fragment.getName());
                        fragmentNameBuffer.append(",");
                    }
                    fragmentNameBuffer.deleteCharAt(fragmentNameBuffer.length()-1);

                    StatManager statManager = FrameworkService.getInstance().getService(StatManager.class);
                    HashMap<String, String> data = new HashMap<String, String>();
                    data.put(StatManager.Constant.KEY_FRAGMENT_NAME, fragmentNameBuffer.toString());
                    statManager.addStat(StatManager.Constant.KEY_ACT_HAS_DESTROY_WHEN_PUSH_FRAGMENT, data);
                } else {
                    for (int i=0; i<fragmentList.length; ++i){
                        ((BaseActivity) baseActivity).pushFragment(fragmentList[i], modes[i]);
                    }
                }
            }
        }else if (hostActivity != null) {
            //cache Fragment
            String[] fragmentNames = new String[fragmentList.length];
            for (int i = 0; i < fragmentList.length; i++ ){
                gSwitchCache.put(fragmentList[i].getName(), fragmentList[i]);
                fragmentNames[i] = fragmentList[i].getName();
                fragmentList[i].setUseAnim(false);
            }

            Intent intent = new Intent(Intent.ACTION_VIEW);
            intent.setClass(mEnv.getApplicationContext(), hostActivity);
            intent.putExtra(BaseActivity.INTENT_EXTRA_FRAGMENT_TAG, fragmentNames);
            intent.putExtra(BaseActivity.INTENT_EXTRA_LAUNCHER_MODE, modes);
            intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);

            mEnv.getApplicationContext().startActivity(intent);
        }else {
            throw new RuntimeException("Framework can't handle startFragment: baseActivity == null && hostActivity == null");
        }
    }

    private void pushFragmentWithFlag(Activity baseActivity, BaseFragment fragment, int intentFlag){
        Class hostActivity = fragment.getHostActivity();
        if(hostActivity == null){
            hostActivity = baseActivity.getClass();
        }

        //cache Fragment
        gSwitchCache.put(fragment.getName(), fragment);

        Intent intent = new Intent(Intent.ACTION_VIEW);
        intent.setClass(baseActivity, hostActivity);
        intent.putExtra(BaseActivity.INTENT_EXTRA_FRAGMENT_TAG,new String[]{fragment.getName()});
        intent.putExtra(BaseActivity.INTENT_EXTRA_LAUNCHER_MODE,new int[]{BaseFragment.LAUNCHER_MODE_NORMAL});
        intent.setFlags(intentFlag);

        baseActivity.startActivity(intent);
    }

    static BaseFragment popCacheFragment(String fragmentTag) {
        return gSwitchCache.remove(fragmentTag);
    }

    /*package*/
    static FragmentCenter getInstance() {
        return sInstance;
    }

    public void fixFragmentModuleState(String fragmentID) {
        if (TextUtils.isEmpty(fragmentID)) {
            return;
        }

        String moduleID = getHostModuleID(fragmentID);
        IModuleEntry moduleEntry = getModuleEntry(moduleID);
        if (moduleEntry!=null) {
            moduleEntry.loadModuleSyn();
        }
    }
}
